#!/usr/bin/python
# coding=utf-8
from debug import log_method_calls, WARNING, DEBUG, RELEASE

try:
    from API_v_1_0 import *
except:  # ImportError is not defined in sandbox
    pass


class Event:
    """
    Simple event realisation that calls subscribed objects with given arguments;
    """
    def __init__(self):
        self.__delegates = []

    def add_callback(self, callback):
        self.__delegates.append(callback)

    def del_callback(self, callback):
        # TODO: implement when needed
        pass

    def fire(self, *args):
        for delegate in self.__delegates:
            try:
                delegate(*args)
            except:
                print 'Exception raised while calling {} with {}'.format(delegate, args)


class WebWorkerBase:
    """
    This is a basic skeleton of webworker that should grab
    information about player's statistics data;
    """
    # override this value in subclass
    API_URL = ''

    def __init__(self):
        self._cache = {}  # you can also use cache in your subclass
        self.eCacheChanged = Event()

    def get_information(self, players):
        """
        Get information about given players; 
        Should make a request to web server (or some other place)
        and return dict[username, dict of key-value pairs of stats data]
        :type players: list[str] 
        :return: None
        """
        pass

    @log_method_calls(WARNING)
    def _get_information(self, request_url):
        web.openUrlAsync(request_url, self._on_receive_information)

    @log_method_calls(DEBUG)
    def _on_receive_information(self, urldata):
        """
        Expects json with players data:
        {
            "data": [
                {
                ...
                "name": "Monstrofil",  <<< THIS FIELD IS REQUIRED
                ...
                }
            ]
        }
        :param urldata: 
        :return: 
        """
        json = utils.jsonDecode(urldata.read())
        updated_players = {info['name']: info for info in json['data']}
        self._cache.update(updated_players)

        self.eCacheChanged.fire(updated_players)


# worker = WebWorkerBase()
# each class that extends base worker should be defined in different file
# and each such file should define global variable 'worker', so it can be easily
# imported in Main.py
